function ULib.registerSvGlobal( _global, metatable_cache, makepairs ) -- We wrap this file in a function so we can receive and manipulate the "real" table.


local function sendFullUpdateToClient( ply, loc )
	loc = loc or {}
	local curT = ULib.global.server
	local rp = ply

	for _, v in ipairs( loc ) do
		curT = curT[ v ]
		if not curT then
			ULib.error( "Invalid table sent via sendFullUpdateToClient!" )
			return
		end
	end

	for key, value in ULib.globalpairs( curT ) do
		local tv = type( value )

		if tv ~= "table" then
			umsg.Start( "ULib_global_server", rp )
				-- We want to pcall all of these because we _must_ call umsg.End() or risk crashing the server.
				local status, err = pcall( ULib.umsgSend, loc )
				if not status then ULib.error( "Error in sendFullUpdateToClient! " .. err ) umsg.End() return end
				local status, err = pcall( ULib.umsgSend, key )
				if not status then ULib.error( "Error in sendFullUpdateToClient! " .. err ) umsg.End() return end
				local status, err = pcall( ULib.umsgSend, value )
				if not status then ULib.error( "Error in sendFullUpdateToClient! " .. err ) umsg.End() return end
			umsg.End()
		else -- We want to separate the table so we avoid the 255 byte limit as much as possible.
			-- Let's create an empty table for them to work off of.
			umsg.Start( "ULib_global_server", rp )
				-- We want to pcall all of these because we _must_ call umsg.End() or risk crashing the server.
				local status, err = pcall( ULib.umsgSend, loc )
				if not status then ULib.error( "Error in sendFullUpdateToClient! " .. err ) umsg.End() return end
				local status, err = pcall( ULib.umsgSend, key )
				if not status then ULib.error( "Error in sendFullUpdateToClient! " .. err ) umsg.End() return end
				local status, err = pcall( ULib.umsgSend, {} )
				if not status then ULib.error( "Error in sendFullUpdateToClient! " .. err ) umsg.End() return end
			umsg.End()

			for k, v in ULib.globalpairs( value ) do
				sendFullUpdateToClient( ply, value.__loc, k, v )
			end
		end
	end
end
hook.Add( "PlayerInitialSpawn", "ULibGlobalUpdate", sendFullUpdateToClient )


local function sendUpdateToClients( loc, key, value )
	local tv = type( value )
	local rp = RecipientFilter()
	rp:AddAllPlayers()

	if tv ~= "table" then
		umsg.Start( "ULib_global_server", rp )
			-- We want to pcall all of these because we _must_ call umsg.End() or risk crashing the server.
			local status, err = pcall( ULib.umsgSend, loc )
			if not status then ULib.error( "Error in sendUpdateToClients! " .. err ) umsg.End() return end
			local status, err = pcall( ULib.umsgSend, key )
			if not status then ULib.error( "Error in sendUpdateToClients! " .. err ) umsg.End() return end
			local status, err = pcall( ULib.umsgSend, value )
			if not status then ULib.error( "Error in sendUpdateToClients! " .. err ) umsg.End() return end
		umsg.End()
	else -- We want to separate the table so we avoid the 255 byte limit as much as possible.
		-- Let's create an empty table for them to work off of.
		umsg.Start( "ULib_global_server", rp )
			-- We want to pcall all of these because we _must_ call umsg.End() or risk crashing the server.
			local status, err = pcall( ULib.umsgSend, loc )
			if not status then ULib.error( "Error in sendUpdateToClients! " .. err ) umsg.End() return end
			local status, err = pcall( ULib.umsgSend, key )
			if not status then ULib.error( "Error in sendUpdateToClients! " .. err ) umsg.End() return end
			local status, err = pcall( ULib.umsgSend, {} )
			if not status then ULib.error( "Error in sendUpdateToClients! " .. err ) umsg.End() return end
		umsg.End()

		for k, v in ULib.globalpairs( value ) do
			sendUpdateToClients( value.__loc, k, v )
		end
	end
end

-- Based off "RecursiveReadOnlyTables" by VeLoSo (http://lua-users.org/wiki/RecursiveReadOnlyTables)

local function make_servergetter( real_table )
	local function getter( t, key )
		local ret = real_table[ key ]
		if type( ret ) == "table" and not metatable_cache[ ret ] then
			real_table[ key ] = ULib.makeServerTable( ret, key, t.__loc )
			ret = real_table[ key ]
		end
		return ret
	end
	return getter
end

local function make_serversetter( real_table )
	local function setter( t, key, value )
		local tv = type( value )

		if tv == "table" then
			value = ULib.makeServerTable( value, key, t.__loc )
		end

		sendUpdateToClients( t.__loc, key, value )

		real_table[ key ] = value
	end
	return setter
end

function ULib.makeServerTable( t, key, loc )
	if loc then
		loc = table.Copy( loc )
	end
	loc = loc or {}
	if key then table.insert( loc, key ) end

	local new={ __loc = loc }
	local mt={
		__metatable = "server table",
		__index = make_servergetter( t ),
		__newindex = make_serversetter( t ),
		__pairs = makepairs( t, ULib.makeServerTable, new ),
		__type = "server table", }
	setmetatable( new, mt )
	metatable_cache[ new ] = mt
	return new
end

ULib.global.server = ULib.makeServerTable( _global.server )

end